# -*- coding: utf-8 -*-
from typing import Optional
from matplotlib import pyplot as plt
import numpy as np
import pandas as pd

from .heat1d import Heat1DModel


def post_process(
    model: Heat1DModel, 
    results: pd.DataFrame,
    sol_tout: tuple,
    sol_tend: tuple,
    where: Optional[slice] = None,
    show: Optional[bool] = False
) -> None:
    """ Display simulation results (temperature, residuals...). """
    where = slice(0, None) if where is None else where
    t, T, dt, r, s = results.iloc[:, :5].to_numpy().T

    T2 = sol_tend[1] - 273.15
    mu2 = np.mean(sol_tend[1][where] - 273.15)

    plt.close("all")
    plt.style.use("seaborn-white")
    fig, ax = plt.subplots(2, 3, figsize=(12, 7))
    ax = ax.ravel()

    ax[0].plot(t, T)
    ax[0].grid(linestyle=":")
    ax[0].set_xlabel("Time [s]")
    ax[0].set_ylabel("Mean temperature [°C]")
    ax[0].set_title(f"Time to target of {t[-1]:.1f} s")

    if sol_tout is not None:
        T1 = sol_tout[1] - 273.15
        mu1 = np.mean(sol_tout[1][where] - 273.15)

        ax[1].plot(model.cell_centers * 1000, T1)
        ax[1].axhline(mu1, color="k")
        ax[1].grid(linestyle=":")
        ax[1].set_xlabel("Coordinate [mm]")
        ax[1].set_ylabel("Temperature [°C]")
        ax[1].set_title(f"Reached {mu1:.0f} °C at {sol_tout[0]:.3f} s")

    ax[2].plot(model.cell_centers * 1000, T2)
    ax[2].axhline(mu2, color="k")
    ax[2].grid(linestyle=":")
    ax[2].set_xlabel("Coordinate [mm]")
    ax[2].set_ylabel("Temperature [°C]")
    ax[2].set_title(f"Reached {mu2:.0f} °C at {sol_tend[0]:.3f} s")

    ax[3].plot(t, 1000 * dt)
    ax[3].grid(linestyle=":")
    ax[3].set_xlabel("Time [s]")
    ax[3].set_ylabel("Time step [x1000 s]")
    ax[3].set_title(f"Time step during integration")

    ax[4].plot(t, r)
    ax[4].grid(linestyle=":")
    ax[4].set_xlabel("Time [s]")
    ax[4].set_ylabel("Residual")
    ax[4].set_title(f"Residuals during integration")

    ax[5].plot(t, s)
    ax[5].grid(linestyle=":")
    ax[5].set_xlabel("Time [s]")
    ax[5].set_ylabel("Sweeps")
    ax[5].set_title(f"Sweeps to converge during integration")

    fig.tight_layout()

    if show:
        plt.show()
